-module(ts_api_session).
-compile(export_all).

-include("ts_db_records.hrl").
-include("yaws_api.hrl").

new(Username) -> 
    DateTime = calendar:now_to_universal_time(erlang:now()),
    Seconds = calendar:datetime_to_gregorian_seconds(DateTime),

    Session = #ts_api_session{
        username = Username,
        expires = Seconds + 600 % timeout is 10 minutes
    },

    CookieVal = yaws_api:new_cookie_session(Session),
    {CookieVal, Session}.

logout(CookieVal) ->
    yaws_api:delete_cookie_session(CookieVal).

get_session(YArg) ->
    % get the cookie header
    Cookie = (YArg#arg.headers)#headers.cookie,

    % get the current server time
    Now = calendar:now_to_universal_time(erlang:now()),
    NowSeconds = calendar:datetime_to_gregorian_seconds(Now),

    % look up the cookie in the session server
    case yaws_api:find_cookie_val("ts_api_session", Cookie) of
    % no cookie, not logged in
    [] -> not_logged_in;

    % found the cookie
    CookieVal -> 
        % get the session data
        case yaws_api:cookieval_to_opaque(CookieVal) of
        {error, _} -> not_logged_in;

        {ok, Session} ->
            if
            % if the cookie has expired
            NowSeconds > Session#ts_api_session.expires ->
                logout(CookieVal),
                session_expired;

            % cookie is fresh
            true -> 
                % update the expiry time
                NewSession = Session#ts_api_session{expires = NowSeconds + 500},
                yaws_api:replace_cookie_session(CookieVal, NewSession),
                
                % return cookie
                NewSession
            end
        end
    end.
