-module(ts_timeline).
-export([create_table/1, new/1, update/1, lookup/2, list/3]).

-include("ts_db_records.hrl").
-include_lib("stdlib/include/qlc.hrl").

create_table(TableOpts) ->
    mnesia:create_table(ts_timeline,
        TableOpts ++ [{attributes, record_info(fields, ts_timeline)},
                      {type, ordered_set}]).

new(TR = #ts_timeline{}) -> ts_common:new(TR).

update(TR = #ts_timeline{}) -> ts_common:update(TR).
    
lookup(Username, TimelineId) ->
    case mnesia:dirty_read(ts_timeline, {Username, TimelineId}) of
        [] -> no_record;
        [Timeline] -> Timeline
    end.

list(Username, Start, Length) -> 
    {atomic, Timelines} = mnesia:transaction(fun() ->
        MatchHead = #ts_timeline{ref = {Username, '_'}, _='_'},
        mnesia:select(ts_timeline, [{MatchHead, [], ['$_']}])
    end),
    lists:sublist(Timelines, Start + 1, Length).
